import { Component, OnInit, ViewChild } from '@angular/core';
import { MatButton } from '@angular/material/button';
import { MatSelect } from '@angular/material/select';
import { MatTableDataSource } from '@angular/material/table';
import { ActivatedRoute, Router } from '@angular/router';
import { FlatService, OverheadAdvanceFlatMappingService, OverheadAdvanceService, PremiseService } from '../data-object-service';
import { Flat, OverheadAdvance, OverheadAdvanceFlatMapping, Premise } from '../data-objects';
import { ExtOverheadAdvanceService } from '../ext-data-object-service';
import { MessageService } from '../message.service';

@Component({
  selector: 'app-flat-details',
  templateUrl: './flat-details.component.html',
  styleUrls: ['./flat-details.component.css']
})
export class FlatDetailsComponent implements OnInit {

  flat: Flat = {
    id: 0,
    description: '',
    premise: 0,
    area: 0.0,
    flat_no: 0
  }

  premise: Premise = {
    id: 0,
    description: '',
    street: '',
    zip: '',
    city: ''
  }

  premises: Premise[]
  mappedOverheadAdvances: OverheadAdvance[]
  allOverheadAdvances: OverheadAdvance[]
  selectedOverheadAdvance: number
  dataSource: MatTableDataSource<OverheadAdvance>
  displayedColumns: string[] = [ "description", "amount", "startdate", "enddate" ]

  @ViewChild('submitButton') submitButton: MatButton
  @ViewChild('mapButton') mapButton: MatButton
  @ViewChild('mapSelect') mapSelect: MatSelect

  constructor(
    private flatService: FlatService,
    private premiseService: PremiseService,
    private extOverheadAdvanceService: ExtOverheadAdvanceService,
    private overheadAdvanceService: OverheadAdvanceService,
    private overheadAdvanceFlatMappingService: OverheadAdvanceFlatMappingService,
    private messageService: MessageService,
    private route: ActivatedRoute,
    private router: Router
  ) { }

  async getFlat(): Promise<void> {
    try {
      const id = +this.route.snapshot.paramMap.get('id')
      if (id != 0) {
        this.flat = await this.flatService.getFlat(id)
        this.premise = await this.premiseService.getPremise(this.flat.premise)
        this.mappedOverheadAdvances = await this.extOverheadAdvanceService.getOverheadAdvancesByFlat(this.flat.id)
        this.dataSource = new MatTableDataSource<OverheadAdvance>(this.mappedOverheadAdvances)
      }
    } catch (err) {
      throw err
      this.messageService.add(JSON.stringify(err, undefined, 4))
    }
  }

  async getPremises(): Promise<void> {
    try {
      this.messageService.add("Trying to load premises")
      this.premises = await this.premiseService.getPremises()
      this.messageService.add("Premises loaded")
    } catch (err) {
      this.messageService.add(JSON.stringify(err, undefined, 4))
    }
  }

  async getOverheadAdvances(): Promise<void> {
    try {
      this.messageService.add("Trying to load overheadadvances")
      this.allOverheadAdvances = await this.overheadAdvanceService.getOverheadAdvances()
      this.messageService.add("overheadadvances loaded")
    } catch (err) {
      this.messageService.add(JSON.stringify(err, undefined, 4))
    }
  }



  async saveFlat() {
    try {
      this.submitButton.disabled = true
      this.messageService.add(`saveFlat: ${ JSON.stringify(this.flat, undefined, 4) }`)
      if (this.flat.id == 0) {
        this.messageService.add("about to insert new flat")
        this.flat = await this.flatService.postFlat(this.flat)
        this.messageService.add(`Successfully added flat with id ${this.flat.id}`)
      } else {
        this.messageService.add("about to update existing flat")
        this.flat = await this.flatService.putFlat(this.flat)
        this.messageService.add(`Successfully changed flat with id ${this.flat.id}`)
      }
      this.router.navigate(['/flats'])
    } finally {
      this.submitButton.disabled = false
    }
  }

  async addOverheadAdvance() {
    try {
      this.mapButton.disabled = true
      this.messageService.add(`addOverheadAdvance: ${ JSON.stringify(this.selectedOverheadAdvance, undefined, 4) }`)
      let newMapping: OverheadAdvanceFlatMapping = {
        'flat': this.flat.id,
        'overhead_advance': this.selectedOverheadAdvance,
        'id': 0
      }
      newMapping = await this.overheadAdvanceFlatMappingService.postOverheadAdvanceFlatMapping(newMapping)
      this.messageService.add(`New overheadadvance flat mapping created: ${newMapping.id}`)
      this.selectedOverheadAdvance = undefined
      this.getFlat()
    } finally {
      this.mapButton.disabled = false
    }
  }

  ngOnInit(): void {
    this.getPremises()
    this.getOverheadAdvances()
    this.getFlat()
  }

}
