import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AccountService, TenantService } from '../data-object-service';
import { Account, Tenant } from '../data-objects';
import { MessageService } from '../message.service';
import { MatButton } from '@angular/material/button';

@Component({
  selector: 'app-tenant-details',
  templateUrl: './tenant-details.component.html',
  styleUrls: ['./tenant-details.component.css']
})
export class TenantDetailsComponent implements OnInit {

  tenant: Tenant = {
    id: 0, 
    salutation: '', 
    firstname: '', 
    lastname: '', 
    address1: '', 
    address2: '', 
    address3: '', 
    zip: '', 
    city: '', 
    phone1: '', 
    phone2: '', 
    iban: '', 
    account: 0
  }

  account: Account = {
    id: 0,
    description: ''
  }

  @ViewChild('submitButton') submitButton: MatButton

  constructor(
    private tenantService: TenantService, 
    private accountService: AccountService,
    private messageService: MessageService,
    private route: ActivatedRoute,
    private router: Router
  ) { }

  async getTenant(): Promise<void> {
    try {
      const id = +this.route.snapshot.paramMap.get('id')
      if (id != 0) {
        this.tenant = await this.tenantService.getTenant(id)
        this.account = await this.accountService.getAccount(this.tenant.account)
      }
    } catch (err) {
      this.messageService.add(JSON.stringify(err, undefined, 4))
    }
  }

  async saveTenant() {
    try {
      this.submitButton.disabled = true
      this.messageService.add("saveTenant")
      this.messageService.add(JSON.stringify(this.tenant, undefined, 4))
      if (this.tenant.id == 0) {
        this.messageService.add("about to insert new tenant")
        this.account = { 
          "id": 0, 
          "description": `account_${this.tenant.firstname}_${this.tenant.lastname}`
        }
        this.account = await this.accountService.postAccount(this.account)
        this.tenant.account = this.account.id
        this.tenant = await this.tenantService.postTenant(this.tenant)
        this.messageService.add(`Successfully added account with id ${this.account.id} and tenant with id ${this.tenant.id}`)
      } else {
        this.messageService.add("about to update existing tenant")
        this.tenant = await this.tenantService.putTenant(this.tenant)
        this.messageService.add(`Successfully changed tenant with id ${this.tenant.id}`)
      }
      this.router.navigate(['/tenants'])
    } finally {
      this.submitButton.disabled = false
    }
  }

  ngOnInit(): void {
    this.getTenant()
  }

}
